//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.plugin.html.guimodel;

import java.net.URL;

import de.ugoe.cs.autoquest.eventcore.guimodel.AbstractDefaultGUIElement;
import de.ugoe.cs.autoquest.eventcore.guimodel.IGUIElementSpec;

/**
 * <p>
 * Base class for all HTML GUI elements.
 * </p>
 * 
 * @version 1.0
 * @author Patrick Harms
 */
public class HTMLGUIElement extends AbstractDefaultGUIElement {

    /**
     * <p>
     * Id for object serialization.
     * </p>
     */
    private static final long serialVersionUID = 1L;

    /**
     * <p>
     * Specification of the GUI Element
     * </p>
     */
    private HTMLGUIElementSpec specification;

    /**
     * <p>
     * Constructor. Creates a new HTMLGUIElement.
     * </p>
     * 
     * @param specification
     *            specification of created GUI element
     * @param parent
     *            parent of the created GUI element; null means that the element is a top-level
     *            window
     */
    public HTMLGUIElement(HTMLGUIElementSpec specification, HTMLGUIElement parent) {
        super(specification, parent);
        this.specification = specification;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.ugoe.cs.autoquest.eventcore.IEventTarget#getPlatform()
     */
    @Override
    public String getPlatform() {
        return "HTML";
    }

    /**
     * <p>
     * Returns web page to which this GUI element belongs
     * </p>
     * 
     * @return the page URL
     */
    URL getPageURL() {
        return specification.getPageURL();
    }

    /**
     * <p>
     * Returns the title of the page to which the GUI element belongs
     * </p>
     * 
     * @return the title
     */
    String getPageTitle() {
        return specification.getPageTitle();
    }

    /**
     * <p>
     * Returns the type of the GUI element, i.e., the name of its tag in HTML
     * </p>
     * 
     * @return the tag in HTML
     */
    String getTagName() {
        return specification.getType();
    }

    /**
     * <p>
     * Returns the id of the tag in HTML
     * </p>
     * 
     * @return the id of the HTML tag
     */
    String getTagId() {
        return specification.getTagId();
    }

    /**
     * <p>
     * Returns the child index of the tag referring to all tags with the same type within the same
     * parent tag 
     * </p>
     * 
     * @return the index
     */
    int getIndex() {
        return specification.getIndex();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * de.ugoe.cs.autoquest.eventcore.guimodel.IGUIElement#updateSpecification(de.ugoe.cs.autoquest.eventcore
     * .guimodel.IGUIElementSpec)
     */
    @Override
    public void updateSpecification(IGUIElementSpec updateSpecification) {
        if (updateSpecification instanceof HTMLGUIElementSpec) {
            specification.update(((HTMLGUIElementSpec) updateSpecification));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String getStringIdentifier() {
        String str = this.toString();
        if (getParent() != null) {
            return getParent().getStringIdentifier() + "->" + str;
        }
        return str;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        String str = getElementDescriptor() + "(" + getTagName();
        if (getTagId() != null) {
            str += "(id=" + getTagId() + ")";
        }
        else {
            str += "[" + getIndex() + "]";
        }
        
        str += ")";
        return str;
    }

    /**
     * <p>
     * A short string describing the GUI element, e.g., Button, Canvas, or ScrollBar.
     * </p>
     * 
     * @return short element descriptor
     */
    protected String getElementDescriptor() {
        return "Default";
    }

}
